#!/usr/bin/perl 
#
# [cruiseid]_ds2elog.pl : selects data from [cruiseid]_ds.db to return to elog
# parameters are --debug --database --key --field
# originally written by Jim Avery whoi
#
# 20110511   lstolp      reworked for r2r elog datasnap, changed file naming,
#                        directory structure and moved variables to top.
#                        as per discussion with andy maffei
#                        currently the color coding of time does not work
#
#

use strict;
use DBI;
use CGI qw(:standard);
use Time::localtime;
use POSIX qw(strftime);
use CGI::Pretty qw( :html3 );
use Getopt::Long;

my $cruiseid = "tn368";
my $elogport = "8090";
my $dsdb = "$cruiseid"."ds.db";
my $dscfg = "$cruiseid"."_ds.cfg";
my $dslogbook = "$cruiseid"."-DS";

my $debug = 0;
my $db_name = '/shipdata/$cruiseid/r2r/eventlog/datasnap/$dsdb';
my $config_file_name = '/shipdata/$cruiseid/r2r/eventlog/datasnap/$dscfg';
my $command =  "/usr/local/bin/elog -h 127.0.0.1 -l $dslogbook -p $elogport -a Author=submit-se";

my $log_book;
my $host;
my $port;

my $greenTime = 60;
my $yellowTime = 120;
my $orangeTime = 180;
my $redTime = 240;

my $attribute = '';
my $username;
my $password;

my $key='';
my $field_no=0;
my $time;
my $epoch_time;
my $config_line;
my $format_string;
my $special_case;
my $data_value;
my $nextPart;
my $nav_degree;
my $nav_minute;
my $nav_letter;
my $ctemp;
my $ftemp;
my $hrsmin_time;
my $hrsmin_hr;
my $hrsmin_min;
my $dbh;
my $table_result;
my $dbi_instr;
my @db_data;
my @data_fields;
my $green_font = "";
my $red_font = "";


GetOptions('debug' => \$debug, 'database=s' => \$db_name, 'config=s', => \$config_file_name,
	'logbook=s' => \$log_book, 'host=s' => \$host, 'logbook=s' => \$log_book, 'port=s' => \$port,
	'green=i' => \$greenTime, 'yellow=i' => \$yellowTime,
	'orange=i' => \$orangeTime, 'red=i' => \$redTime);

$dbh = DBI->connect( "dbi:SQLite:". $db_name, $username, $password, { PrintError => 0} ) || die "Can not connect.";

print STDERR "Database connected: ", $dbh, "\n" if $debug;

$table_result = $dbh->do( "CREATE TABLE latest_data (
              header  VARCHAR(10) PRIMARY KEY ,
              udp_time VARCHAR (10),
              data VARCHAR (180),
              timeEnter VARCHAR(120))" );

if ($debug) {
  if (!defined($table_result)) {
        print STDERR "Database table already exists\n";
  } else {
        print STDERR "Database table created\n";
  }
}


$time = localtime;
$epoch_time = time;

open (config_file, $config_file_name) || die "Can't find config file.";

while ($config_line = <config_file>) {
	chop ($config_line);
        print STDERR "Line from config file: ", join( '|', split(':', $config_line)), "\n" if $debug;   

        ($attribute, $key, $field_no, $format_string, $special_case) = split(':', $config_line);
        $data_value = make_entry();

	$nextPart = ' -a "'.$attribute.'"="'.$data_value.'" ';
	$command = $command.$nextPart;
}

$command = $command.' "  " > /dev/null';

print "Command: ", $command, "\n" if $debug;

system ($command);

sub make_entry {

	print STDERR "Make_entry called:\n attribute $attribute key $key field no $field_no \n",
		"format $format_string case $special_case \n" if $debug;
	
	$dbi_instr = $dbh->prepare("SELECT * FROM latest_data WHERE header = '$key' ");
        $dbi_instr->execute( );

        @db_data = $dbi_instr->fetchrow_array;
        print STDERR "Fetched db_data: $key \n$db_data[0]\n$db_data[1]\n$db_data[2]\n$db_data[3]\n" if $debug;
	@data_fields = split (/,/, $db_data[2]);

	print STDERR "Special case field is: $special_case \n" if $debug;

	if ($special_case eq "") {
		print STDERR "Found normal case\n" if $debug;
		print STDERR join('|',@data_fields), "\n" if $debug;
		print STDERR "Field: $field_no, Data: $data_fields[$field_no]\n" if $debug;
		$data_value = sprintf ( $format_string, $data_fields[$field_no]);
	}
	elsif ($special_case eq "blank") {
		print STDERR "Found blank case\n" if $debug;
		$attribute = "";
		$data_value = "";
	}		
	elsif ($special_case eq "longitude") {
		print STDERR "Found longitude case\n" if $debug;
		$nav_degree = substr($data_fields[$field_no], 0, 3);
		$nav_minute = substr($data_fields[$field_no], 3, 8);
		$nav_letter = $data_fields[$field_no+1];
		$data_value = sprintf ($format_string, $nav_degree, $nav_minute, $nav_letter);
	}
	elsif ($special_case eq "latitude") {
		print STDERR "Found latitude case\n" if $debug;
		$nav_degree = substr($data_fields[$field_no], 0, 2);
		$nav_minute = substr($data_fields[$field_no], 2, 8);
		$nav_letter = $data_fields[$field_no+1];
		$data_value = sprintf ($format_string, $nav_degree, $nav_minute, $nav_letter);
	}
	elsif ($special_case eq "airtemp") {
		print STDERR "Found airtemp case\n" if $debug;
		$ctemp = $data_fields[$field_no];
		$ftemp = 32.0 +( 9.0 * $ctemp) / 5.0;
		$data_value = sprintf ($format_string, $ctemp, $ftemp);
	}
	elsif ($special_case eq "hrsmin") {
		print STDERR "Found hrsmin case\n" if $debug;
		$hrsmin_time = $data_fields[$field_no];
		$hrsmin_hr = int $hrsmin_time;
		$hrsmin_min = 60. * ($hrsmin_time - $hrsmin_hr);
		$format_string = join (':', split (';', $format_string));
		if ($hrsmin_time < 0.05) {
			$format_string = $format_string."*";
		}
		$data_value = sprintf($format_string, $hrsmin_hr, $hrsmin_min);
		if ($hrsmin_time < 0.) {
			$data_value = "n/a";
		}
	}
	elsif ($special_case eq "netconn") {
		print STDERR "Found netconn case\n" if $debug;
		$data_value = $data_fields[$field_no];
		$data_value = ($data_value eq "Connected") 
					? $green_font.$data_value
					: $red_font.$data_value;
	}

	print STDERR "Data value, format string, data:\n$data_value $format_string $data_fields[$field_no]\n" if $debug;

#	$age = $epoch_time - $db_data[3];
#	$fstart = $age > 900
#		? $red_font
#		  : $age > 300
#       	            ? $orange_font
#       	              : $age > 30
#       	                ? $yellow_font
#       	                  : $green_font;
#	print STDERR "Calculated age and font $age $fstart \n" if $debug;


#	if (($age > 900) && ($data_value ne "")) {
#		print STDERR "Old, non-blank data: Age: $age Data value: $data_value\n" if $debug;
#		$data_value = "No data";
#	}
	print STDERR "At make_entry return:\nAttribute: $attribute Data: $data_value\n" if $debug;
	return ($data_value);
}
